package proxytemplate

import (
	. "github.com/onsi/ginkgo/v2"
	. "github.com/onsi/gomega"

	config_core "github.com/kumahq/kuma/pkg/config/core"
	. "github.com/kumahq/kuma/test/framework"
	"github.com/kumahq/kuma/test/framework/client"
)

var universalCluster Cluster

var _ = E2EBeforeSuite(func() {
	universalCluster = NewUniversalCluster(NewTestingT(), Kuma1, Silent)

	err := NewClusterSetup().
		Install(Kuma(config_core.Standalone)).
		Install(TestServerUniversal("test-server", "default", WithArgs([]string{"echo", "--instance", "echo-v1"}))).
		Install(DemoClientUniversal(AppModeDemoClient, "default", WithTransparentProxy(true))).
		Setup(universalCluster)
	Expect(err).ToNot(HaveOccurred())
})

var _ = E2EAfterSuite(func() {
	Expect(universalCluster.DismissCluster()).To(Succeed())
})

func ProxyTemplateUniversal() {
	It("should add a header using Lua filter", func() {
		// given
		proxyTemplate := `
type: ProxyTemplate
mesh: default
name: backend-lua-filter
selectors:
  - match:
      kuma.io/service: demo-client
conf:
  imports:
    - default-proxy # apply modifications on top of resources generated by Kuma
  modifications:
    - httpFilter:
        operation: addBefore
        match:
          name: envoy.filters.http.router
          origin: outbound
          listenerTags:
            kuma.io/service: test-server
        value: |
          name: envoy.filters.http.lua
          typedConfig:
            '@type': type.googleapis.com/envoy.extensions.filters.http.lua.v3.Lua
            inline_code: |
              function envoy_on_request(request_handle)
                request_handle:headers():add("X-Header", "test")
              end
`

		// when
		err := YamlUniversal(proxyTemplate)(universalCluster)

		// then
		Expect(err).ToNot(HaveOccurred())
		Eventually(func(g Gomega) {
			responses, err := client.CollectResponses(universalCluster, "demo-client", "test-server.mesh")
			g.Expect(err).ToNot(HaveOccurred())
			g.Expect(responses[0].Received.Headers["X-Header"]).To(ContainElements("test"))
		}, "30s", "1s").Should(Succeed())
	})
}
